/*
	copyright 2006 Gilles GIGAN (gilles.gigan@gmail.com)
			
	This file is part of libv4l2.

   	libv4l2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    libv4l2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef H_V4L2_INPUT
#define H_V4L2_INPUT

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <asm/types.h>		//for videodev2
#include <linux/videodev2.h>
#include <string.h>


#include "frame.h"

#define CLEAR(x) memset(&x, 0x0, sizeof(x));
		
enum STANDARDS {WEBCAM, PAL, SECAM, NTSC};

struct mmap_buffer {
	int length;
	void *start;
};

struct mmap {
	int req_buffer_nr;
	int buffer_nr;		//actual number of mmap buffers
	struct mmap_buffer *buffers;
	struct v4l2_buffer tmp_b;
};

struct control_list {
	int count;
	struct v4l2_queryctrl *ctrl;
};

struct capture_device {
	int fd;
	int width;
	int height;
	int fps;			//not implemented yet
	int std;			//v4l2 standard
	int channel;		//channel number (for video capture cards, not webcams)
	char file[100];		//device file name
	int bytesperline;	//number of bytes per line in the captured image
	int imagesize;
	struct mmap *mmap;
	struct control_list *ctrls;
};

//Capture methods
struct capture_device *init_libv4l2(char *, int, int, int, int, int);
int open_device(struct capture_device *);
int check_capture_capabilities(struct capture_device *);
int set_cap_param(struct capture_device *);
int init_capture(struct capture_device *);
int start_capture(struct capture_device *);
int wait_for_frame(struct capture_device *);

//EITHER
struct frame *get_frame(struct capture_device *);
//OR
struct v4l2_buffer *dequeue_buffer(struct capture_device *);
void *get_frame_buffer(struct capture_device *, struct v4l2_buffer *, int *);
void enqueue_buffer(struct capture_device *, struct v4l2_buffer *);
 
//
int stop_capture(struct capture_device *);
void free_capture(struct capture_device *);
void close_device(struct capture_device *);
void del_libv4l2(struct capture_device *);

//Control methods
struct control_list *list_control(struct capture_device *);
int get_control_value(struct capture_device *, struct v4l2_queryctrl *);
void set_control_value(struct capture_device *, struct v4l2_queryctrl *,  int);
void free_control_list(struct control_list *);

//Query and list methods (printf to stdout)
void query_frame_sizes(struct capture_device *);
void list_cap(struct capture_device *);
void query_capture_intf(struct capture_device *);
void query_current_image_fmt(struct capture_device *);


#endif
